# -*- coding: iso-8859-1 -*-
"""
Spiele mit Zipf's Law

Vgl. Skript: "Der Good-Turing-Schtzer I"
"""

from __future__ import generators

import re

# Python 2.2 compatibility
try:
	enumerate(range(2))
except NameError:
	def enumerate(seq):
		ind = 0
		for el in seq:
			yield ind, el
			ind += 1

def tokenize(tx):
	"""returns a list of individual words (defined by a regular expression)
	present in the text tx.
	"""
	return re.sub(r"(?L)[^\w\s']", "", 
			re.sub("(?s)<[^>]*>", "", tx.lower())).split()

def getWordDistribution(tx):
	"""returns a dictionary for word frequencies in the text tx.
	"""
	freqDict = {}
	for word in tokenize(tx):
		freqDict[word] = freqDict.get(word, 0)+1
	return freqDict

def getFoFDistribution(tx):
	"""returns an (unnormalized) distribution of frequencies of frequencies
	of the words in the text tx.
	"""
	freqDist = {}
	wDist = getWordDistribution(tx)
	for freq in wDist.itervalues():
		freqDist[freq] = freqDist.get(freq, 0)+1
	return freqDist

def _getSortedItemStrs(dict):
	"""returns a key-sorted list of strings reprenting the items in
	dict.
	"""
	keys = dict.keys()
	keys.sort()
	strList = ["%s: %s"%(str(key), str(dict[key])) 
		for key in keys]
	return strList

def prettyPrintDist(dist, scrWid=80):
	"""prints a "distribution" (i.e., a dictionary) sorted according
	to key and in columns for an scrWid-col screen.
	>>> prettyPrintDist({1:"bla", 2: -1, -1:"goggle", 300000:1e17})
	-1: goggle     1: bla         2: -1          300000: 1e+17  
	>>> prettyPrintDist(dict([(n, "a"*n) for n in range(20)]))
	0:                       7: aaaaaaa               14: aaaaaaaaaaaaaa       
	1: a                     8: aaaaaaaa              15: aaaaaaaaaaaaaaa      
	2: aa                    9: aaaaaaaaa             16: aaaaaaaaaaaaaaaa     
	3: aaa                   10: aaaaaaaaaa           17: aaaaaaaaaaaaaaaaa    
	4: aaaa                  11: aaaaaaaaaaa          18: aaaaaaaaaaaaaaaaaa   
	5: aaaaa                 12: aaaaaaaaaaaa         19: aaaaaaaaaaaaaaaaaaa  
	6: aaaaaa                13: aaaaaaaaaaaaa        
	"""
	strList = _getSortedItemStrs(dist)
	colWid = max([len(s) for s in strList])+2
	numCol = max(1, scrWid/colWid)
	colLen = len(strList)/numCol
	if len(strList)%numCol:
		colLen += 1
	fmtStr = "%%-%ds"%colWid
	output = [[] for i in range(colLen)]
	for col in range(numCol):
		for ind, val in enumerate(strList[colLen*col:colLen*(col+1)]):
			output[ind].append(fmtStr%val)
	print "\n".join(["".join(l) for l in output])

def checkRankLaw(dist):
	freqs = dist.keys()
	freqs.sort()
	freqs.reverse()
	newDist = {}
	for rank, freq in enumerate(freqs):
		newDist[rank+1] = (rank+1)*freq
	prettyPrintDist(newDist)

def checkFreqLaw(dist):
	items = dist.items()
	items.sort()
	items.reverse()
	newDist = {}
	rank = 1
	for freq, freqCount in items:
		newDist[rank] = freqCount*freq**1.7
		rank += 1
	prettyPrintDist(newDist)

def printGnuplotFile(xLabel, yLabel, points):
	"""prints a gnuplot command file that draws a graph of the points
	with axis markings xLabel and yLabel.
	"""
	plotdata = "\n".join(["%s %s"%(str(k), str(v))
		for k, v in points])+"\ne"
	print ('set xlabel "%(xLabel)s"\n'
		'set ylabel "%(yLabel)s"\n'
		'set pointsize 3\n'
		'set nokey\n'
		'plot "-" with points pointtype 4\n%(plotdata)s\n'
	)%locals()


def _test():
	import doctest, checkZipf
	return doctest.testmod(checkZipf)


if __name__=="__main__":
	_test()
	import sys
	from math import log
	fofDist = getFoFDistribution(open(sys.argv[1]).read())

	prettyPrintDist(fofDist)
#	checkRankLaw(fofDist)
#	checkFreqLaw(fofDist)
#	printGnuplotFile("n_r", "r", 
#		[(r, nr) for r, nr in fofDist.items()])
#	printGnuplotFile("log r", "log n_r", 
#		[(log(r), log(nr)) for r, nr in fofDist.items()])
